package sun.management.snmp.jvminstr;

import java.util.TreeMap;
import sun.management.snmp.util.SnmpCachedData;
import java.lang.management.ThreadInfo;
import com.sun.jmx.mbeanserver.Util;
import java.util.Map;
import sun.management.snmp.util.JvmContextFactory;
import sun.management.snmp.util.SnmpTableHandler;
import com.sun.jmx.snmp.SnmpStatusException;
import com.sun.jmx.snmp.agent.SnmpStandardObjectServer;
import com.sun.jmx.snmp.agent.SnmpMib;
import com.sun.jmx.snmp.SnmpOid;
import sun.management.snmp.util.MibLogger;
import sun.management.snmp.util.SnmpTableCache;
import sun.management.snmp.jvmmib.JvmThreadInstanceTableMeta;

public class JvmThreadInstanceTableMetaImpl extends JvmThreadInstanceTableMeta
{
    static final long serialVersionUID = -8432271929226397492L;
    public static final int MAX_STACK_TRACE_DEPTH = 0;
    protected SnmpTableCache cache;
    static final MibLogger log;
    
    static SnmpOid makeOid(long n) {
        return new SnmpOid(new long[] { n >> 56 & 0xFFL, n >> 48 & 0xFFL, n >> 40 & 0xFFL, n >> 32 & 0xFFL, n >> 24 & 0xFFL, n >> 16 & 0xFFL, n >> 8 & 0xFFL, n & 0xFFL });
    }
    
    static long makeId(SnmpOid snmpOid) {
        long n = 0L;
        long[] longValue = snmpOid.longValue(false);
        return n | longValue[0] << 56 | longValue[1] << 48 | longValue[2] << 40 | longValue[3] << 32 | longValue[4] << 24 | longValue[5] << 16 | longValue[6] << 8 | longValue[7];
    }
    
    public JvmThreadInstanceTableMetaImpl(SnmpMib snmpMib, SnmpStandardObjectServer snmpStandardObjectServer) {
        super(snmpMib, snmpStandardObjectServer);
        this.cache = new JvmThreadInstanceTableMetaImpl.JvmThreadInstanceTableCache(this, ((JVM_MANAGEMENT_MIB_IMPL)snmpMib).validity());
        JvmThreadInstanceTableMetaImpl.log.debug("JvmThreadInstanceTableMetaImpl", "Create Thread meta");
    }
    
    protected SnmpOid getNextOid(Object o) throws SnmpStatusException {
        JvmThreadInstanceTableMetaImpl.log.debug("JvmThreadInstanceTableMetaImpl", "getNextOid");
        return this.getNextOid(null, o);
    }
    
    protected SnmpOid getNextOid(SnmpOid next, Object o) throws SnmpStatusException {
        JvmThreadInstanceTableMetaImpl.log.debug("getNextOid", "previous=" + next);
        SnmpTableHandler handler = this.getHandler(o);
        if (handler == null) {
            JvmThreadInstanceTableMetaImpl.log.debug("getNextOid", "handler is null!");
            throw new SnmpStatusException(224);
        }
        do {
            next = handler.getNext(next);
            if (next == null) {
                break;
            }
            continue;
        } while (this.getJvmThreadInstance(o, next) == null);
        JvmThreadInstanceTableMetaImpl.log.debug("*** **** **** **** getNextOid", "next=" + next);
        if (next == null) {
            throw new SnmpStatusException(224);
        }
        return next;
    }
    
    protected boolean contains(SnmpOid snmpOid, Object o) {
        SnmpTableHandler handler = this.getHandler(o);
        return handler != null && handler.contains(snmpOid) && this.getJvmThreadInstance(o, snmpOid) != null;
    }
    
    public Object getEntry(SnmpOid snmpOid) throws SnmpStatusException {
        JvmThreadInstanceTableMetaImpl.log.debug("*** **** **** **** getEntry", "oid [" + snmpOid + "]");
        if (snmpOid == null || snmpOid.getLength() != 8) {
            JvmThreadInstanceTableMetaImpl.log.debug("getEntry", "Invalid oid [" + snmpOid + "]");
            throw new SnmpStatusException(224);
        }
        Map<Object, Object> userData = JvmContextFactory.getUserData();
        SnmpTableHandler handler = this.getHandler(userData);
        if (handler == null || !handler.contains(snmpOid)) {
            throw new SnmpStatusException(224);
        }
        JvmThreadInstanceEntryImpl jvmThreadInstance = this.getJvmThreadInstance(userData, snmpOid);
        if (jvmThreadInstance == null) {
            throw new SnmpStatusException(224);
        }
        return jvmThreadInstance;
    }
    
    protected SnmpTableHandler getHandler(Object x) {
        Map<String, SnmpTableHandler> map;
        if ((x instanceof Map)) {
            map = (Map)Util.cast(x);
        }
        else {
            map = null;
        }
        if (map != null) {
            SnmpTableHandler snmpTableHandler = (SnmpTableHandler)map.get("JvmThreadInstanceTable.handler");
            if (snmpTableHandler != null) {
                return snmpTableHandler;
            }
        }
        SnmpTableHandler tableHandler = this.cache.getTableHandler();
        if (map != null && tableHandler != null) {
            map.put("JvmThreadInstanceTable.handler", tableHandler);
            return tableHandler;
        }
        return tableHandler;
    }
    
    private ThreadInfo getThreadInfo(long n) {
        return JvmThreadingImpl.getThreadMXBean().getThreadInfo(n, 0);
    }
    
    private ThreadInfo getThreadInfo(SnmpOid snmpOid) {
        return this.getThreadInfo(makeId(snmpOid));
    }
    
    private JvmThreadInstanceEntryImpl getJvmThreadInstance(Object x, SnmpOid snmpOid) {
        JvmThreadInstanceEntryImpl jvmThreadInstanceEntryImpl = null;
        String string = null;
        Map<String, Object> map = null;
        int debugOn = JvmThreadInstanceTableMetaImpl.log.isDebugOn();
        if ((x instanceof Map)) {
            map = (Map)Util.cast(x);
            string = "JvmThreadInstanceTable.entry." + snmpOid.toString();
            jvmThreadInstanceEntryImpl = (JvmThreadInstanceEntryImpl)map.get(string);
        }
        if (jvmThreadInstanceEntryImpl != null) {
            if (!(debugOn == 0)) {
                JvmThreadInstanceTableMetaImpl.log.debug("*** getJvmThreadInstance", "Entry found in cache: " + string);
                return jvmThreadInstanceEntryImpl;
            }
            return jvmThreadInstanceEntryImpl;
        }
        else {
            if (!(debugOn == 0)) {
                JvmThreadInstanceTableMetaImpl.log.debug("*** getJvmThreadInstance", "Entry [" + snmpOid + "] is not in cache");
            }
            ThreadInfo threadInfo = null;
            try {
                threadInfo = this.getThreadInfo(snmpOid);
            }
            catch (RuntimeException ex) {
                JvmThreadInstanceTableMetaImpl.log.trace("*** getJvmThreadInstance", "Failed to get thread info for rowOid: " + snmpOid);
                JvmThreadInstanceTableMetaImpl.log.debug("*** getJvmThreadInstance", ex);
            }
            if (threadInfo == null) {
                if (!(debugOn == 0)) {
                    JvmThreadInstanceTableMetaImpl.log.debug("*** getJvmThreadInstance", "No entry by that oid [" + snmpOid + "]");
                }
                return null;
            }
            else {
                JvmThreadInstanceEntryImpl o = new JvmThreadInstanceEntryImpl(threadInfo, snmpOid.toByte());
                if (map != null) {
                    map.put(string, o);
                }
                if (!(debugOn == 0)) {
                    JvmThreadInstanceTableMetaImpl.log.debug("*** getJvmThreadInstance", "Entry created for Thread OID [" + snmpOid + "]");
                    return o;
                }
                return o;
            }
        }
    }
    
    static {
        log = new MibLogger(JvmThreadInstanceTableMetaImpl.class);
    }
    
    private static class JvmThreadInstanceTableCache extends SnmpTableCache
    {
        static final long serialVersionUID = 4947330124563406878L;
        private final JvmThreadInstanceTableMetaImpl meta;
        
        JvmThreadInstanceTableCache(JvmThreadInstanceTableMetaImpl meta, long validity) {
            this.validity = validity;
            this.meta = meta;
        }
        
        public SnmpTableHandler getTableHandler() {
            return this.getTableDatas(JvmContextFactory.getUserData());
        }
        
        protected SnmpCachedData updateCachedDatas(Object o) {
            long[] allThreadIds = JvmThreadingImpl.getThreadMXBean().getAllThreadIds();
            long currentTimeMillis = System.currentTimeMillis();
            SnmpOid[] array = new SnmpOid[allThreadIds.length];
            TreeMap treeMap = new TreeMap(SnmpCachedData.oidComparator);
            for (int i = 0; i < allThreadIds.length; ++i) {
                JvmThreadInstanceTableMetaImpl.log.debug("", "Making index for thread id [" + allThreadIds[i] + "]");
                SnmpOid oid = JvmThreadInstanceTableMetaImpl.makeOid(allThreadIds[i]);
                treeMap.put(oid, oid);
            }
            return new SnmpCachedData(currentTimeMillis, treeMap);
        }
    }
}
